@@ -1,6 +1,10 @@
 /*
- *  Copyright (C) 2001-2002 Hewlett-Packard Co.
+ *  Copyright (C) 2001-2003 Hewlett-Packard Co.
  *	Contributed by Stephane Eranian <eranian@hpl.hp.com>
+ *  Copyright (C) 2006-2009 Intel Corporation
+ *	Contributed by Fenghua Yu <fenghua.yu@intel.com>
+ *	Contributed by Bibo Mao <bibo.mao@intel.com>
+ *	Contributed by Chandramouli Narayanan <mouli@linux.intel.com>
  *
  * This file is part of the ELILO, the EFI Linux boot loader.
  *
@@ -32,8 +36,8 @@
 
 #define FS_NAME L"netfs"
 
-#define NETFS_DEFAULT_BUFSIZE		4*MB
-#define NETFS_DEFAULT_BUFSIZE_INC	1*MB
+#define NETFS_DEFAULT_BUFSIZE		16*MB
+#define NETFS_DEFAULT_BUFSIZE_INC	 8*MB
 
 #define NETFS_DEFAULT_SERVER_TYPE	EFI_PXE_BASE_CODE_BOOT_TYPE_BOOTSTRAP
 #define NETFS_FD_MAX	2
@@ -231,25 +235,29 @@ netfs_extract_ip(netfs_priv_state_t *nfs)
 		/*
 		 * XXX: TFTPD server not quite right when using PXE, need to extract bootservers...
 		 */
-	VERB_PRT(1, Print(L"Local IP: %d.%d.%d.%d SM: %d.%d.%d.%d TFTPD IP: %d.%d.%d.%d\n", 
-		pxe->Mode->StationIp.v4.Addr[0],
-		pxe->Mode->StationIp.v4.Addr[1],
-		pxe->Mode->StationIp.v4.Addr[2],
-		pxe->Mode->StationIp.v4.Addr[3],
-		pxe->Mode->SubnetMask.v4.Addr[0],
-		pxe->Mode->SubnetMask.v4.Addr[1],
-		pxe->Mode->SubnetMask.v4.Addr[2],
-		pxe->Mode->SubnetMask.v4.Addr[3],
-		nfs->srv_ip.v4.Addr[0], 
-		nfs->srv_ip.v4.Addr[1], 
-		nfs->srv_ip.v4.Addr[2], 
-		nfs->srv_ip.v4.Addr[3]));
+	VERB_PRT(1, Print(L"Local IP: %d.%d.%d.%d\n",
+		pxe->Mode->StationIp.v4.Addr[0] & 0xff,
+		pxe->Mode->StationIp.v4.Addr[1] & 0xff,
+		pxe->Mode->StationIp.v4.Addr[2] & 0xff,
+		pxe->Mode->StationIp.v4.Addr[3] & 0xff));
+
+	VERB_PRT(1, Print(L"SM: %d.%d.%d.%d\n",
+		pxe->Mode->SubnetMask.v4.Addr[0] & 0xff,
+		pxe->Mode->SubnetMask.v4.Addr[1] & 0xff,
+		pxe->Mode->SubnetMask.v4.Addr[2] & 0xff,
+		pxe->Mode->SubnetMask.v4.Addr[3] & 0xff));
+
+	VERB_PRT(1, Print(L"TFTPD IP: %d.%d.%d.%d\n", 
+		nfs->srv_ip.v4.Addr[0] & 0xff, 
+		nfs->srv_ip.v4.Addr[1] & 0xff, 
+		nfs->srv_ip.v4.Addr[2] & 0xff, 
+		nfs->srv_ip.v4.Addr[3] & 0xff));
 
 	VERB_PRT(1, Print(L"Gateway IP: %d.%d.%d.%d\n",
-			nfs->gw_ip.v4.Addr[0], 
-			nfs->gw_ip.v4.Addr[1], 
-			nfs->gw_ip.v4.Addr[2], 
-			nfs->gw_ip.v4.Addr[3]));
+			nfs->gw_ip.v4.Addr[0] & 0xff, 
+			nfs->gw_ip.v4.Addr[1] & 0xff, 
+			nfs->gw_ip.v4.Addr[2] & 0xff, 
+			nfs->gw_ip.v4.Addr[3] & 0xff));
 
 }
 
@@ -258,7 +266,7 @@ netfs_start(EFI_PXE_BASE_CODE *pxe)
 {
 	EFI_STATUS status;
 
-	status = pxe->Start(pxe, FALSE);
+	status = uefi_call_wrapper(pxe->Start, 2, pxe, FALSE);
 	if (EFI_ERROR(status)) return status;
 
 	return pxe->Dhcp(pxe, FALSE);
@@ -271,6 +279,7 @@ netfs_open(netfs_interface_t *this, CHAR16 *name, UINTN *fd)
 	netfs_fd_t	   *f;
 	EFI_STATUS	   status;
 	CHAR8		   ascii_name[FILENAME_MAXLEN];
+	UINTN 		   blocksize = 0, prev_netbufsize;
 
 	if (this == NULL || name == NULL || fd == NULL) return EFI_INVALID_PARAMETER;
 
@@ -314,19 +323,62 @@ netfs_open(netfs_interface_t *this, CHAR16 *name, UINTN *fd)
 retry:
 	f->netbuf_size = f->netbuf_maxsize;
 
-	status = nfs->pxe->Mtftp(nfs->pxe, EFI_PXE_BASE_CODE_TFTP_READ_FILE, f->netbuf, FALSE,
-			    &f->netbuf_size, NULL, &nfs->srv_ip, ascii_name, NULL, FALSE);
+	DBG_PRT((L"\nbefore netbuf:" PTR_FMT " netbuf_size=%d\n", f->netbuf, f->netbuf_size));
 
+	/* 
+	 * For EFI versions older than 14.61:
+	 *   it seems like there is an EFI bug (or undocumented behavior) when the buffer size
+	 *   is too small AND the blocksize parameter is NULL, i.e., used the largest possible.
+	 *   In this case, Mtftp() never returns EFI_BUFFER_TOO_SMALL but EFI_TIMEOUT instead.
+	 *   This is true for 1.02 and also 1.10 it seems. Here we set it to the minimal value (512).
+	 *
+	 *   Also it seems like on a READ_FILE which returns EFI_BUFFER_TOO_SMALL, the buffersize
+	 *   is NOT updated to reflect the required size for the next attempt.
+	 *
+	 * For EFI versions 14.61 and higher:
+	 *  In case the buffer is too small AND the TFTP server reports the file size (see RFC 2349), 
+	 *  the f->netbuf_size will report the exact size for the buffer.
+	 */
+	prev_netbufsize = f->netbuf_size;
+
+	status = uefi_call_wrapper(nfs->pxe->Mtftp, 10, nfs->pxe, EFI_PXE_BASE_CODE_TFTP_READ_FILE, f->netbuf, FALSE,
+			    &(f->netbuf_size), 
+			    blocksize > 0 ? &blocksize : NULL, 
+			    &nfs->srv_ip, 
+			    ascii_name, 
+			    NULL, 
+			    FALSE);
+
+	DBG_PRT((L"after Mftp=%r netbuf:" PTR_FMT " netbuf_size=%d blocksize=%d\n", 
+		status, 
+		f->netbuf, 
+		f->netbuf_size, 
+		blocksize));
+
+	if (status == EFI_TIMEOUT && blocksize == 0) {
+		/*
+		 * XXX: if blocksize is not adjusted we could loop forever here
+		 */
+		//blocksize = 512;
+		status = EFI_BUFFER_TOO_SMALL;
+	}
 	/*
 	 * check if we need to increase our buffer size
 	 */
 	if (status == EFI_BUFFER_TOO_SMALL) {
-		ERR_PRT((L"buffer too small, need netbuf_size=%d", f->netbuf_size));
-#ifdef EFI_AT_LEAST_12_35
-		f->netbuf_maxsize = f->netbuf_size ? f->netbuf_size : f->netbuf_maxsize + NETFS_DEFAULT_BUFSIZE_INC;
-#else
-		f->netbuf_maxsize += NETFS_DEFAULT_BUFSIZE_INC;
-#endif
+		DBG_PRT((L"buffer too small, need netbuf_size=%d", f->netbuf_size));
+		/*
+		 * if the TFTP server supports TFTP options, then we should
+		 * get the required size. So we test to see if the size
+		 * we set has changed. If so, we got the required size.
+		 * If not, we increase the buffer size and retry.
+		 */
+		if (f->netbuf_size == prev_netbufsize) {
+			f->netbuf_maxsize += NETFS_DEFAULT_BUFSIZE_INC;
+		} else {
+			/* we got an answer from the TFTP server, let's try it */
+			f->netbuf_maxsize = f->netbuf_size;
+		}
 		free(f->netbuf);
 
 		f->netbuf = NULL; /* will force reallocation */
@@ -349,7 +401,7 @@ retry:
 		VERB_PRT(2, Print(L"Done\n"));
 	} else {
 		netfs_fd_free(nfs, f);
-		VERB_PRT(2, Print(L"Failed\n"));
+		VERB_PRT(2, Print(L"Failed: %r\n", status));
 	}
 	DBG_PRT((L"File %s netbuf_size=%d: %r", name, f->netbuf_size, status));
 #if 0
@@ -490,7 +542,6 @@ netfs_getinfo(netfs_interface_t *this, netfs_info_t *info)
 	CHAR8 str[256];
 	INTN len, r;
 
-
 	if (this == NULL || info == NULL) return EFI_INVALID_PARAMETER;
 
 	nfs = FS_PRIVATE(this);
@@ -519,6 +570,14 @@ netfs_getinfo(netfs_interface_t *this, netfs_info_t *info)
 
 	VERB_PRT(3, Print(L"hostname(12): %a\n", str));
 
+	/*
+	 * extract bootfile name from DHCP exchanges
+	 */
+	if (nfs->using_pxe == 0) {
+ 		ascii2U(nfs->pxe->Mode->DhcpAck.Dhcpv4.BootpBootFile, info->bootfile, NETFS_BOOTFILE_MAXLEN);
+		VERB_PRT(3, Print(L"bootfile: %s\n", info->bootfile));
+	}
+
 skip_options:
 	return EFI_SUCCESS;
 }
@@ -572,7 +631,7 @@ netfs_query_layer(netfs_interface_t *this, UINT16 server_type, UINT16 layer, UIN
 
 	if (server_type == 0) server_type = find_pxe_server_type(nfs->pxe);
 
-	status = nfs->pxe->Discover(nfs->pxe, server_type, &layer, FALSE, 0);
+	status = uefi_call_wrapper(nfs->pxe->Discover, 5, nfs->pxe, server_type, &layer, FALSE, 0);
 	if(status == EFI_SUCCESS) {
 		ascii2U(nfs->pxe->Mode->PxeReply.Dhcpv4.BootpBootFile, str, maxlen);
 	} 
@@ -625,13 +684,13 @@ netfs_install_one(EFI_HANDLE dev, VOID **intf)
 	netfs_t *netfs;
 	EFI_PXE_BASE_CODE *pxe;
 
-	status = BS->HandleProtocol (dev, &NetFsProtocol, (VOID **)&netfs);
+	status = uefi_call_wrapper(BS->HandleProtocol, 3, dev, &NetFsProtocol, (VOID **)&netfs);
 	if (status == EFI_SUCCESS) {
 		ERR_PRT((L"Warning: found existing %s protocol on device", FS_NAME));
 		goto found;
 	}
 	
-	status = BS->HandleProtocol (dev, &PxeBaseCodeProtocol, (VOID **)&pxe);
+	status = uefi_call_wrapper(BS->HandleProtocol, 3, dev, &PxeBaseCodeProtocol, (VOID **)&pxe);
 	if (EFI_ERROR(status)) return EFI_INVALID_PARAMETER;
 
 
@@ -672,7 +731,7 @@ netfs_install(VOID)
 	EFI_STATUS status;
 	VOID *intf;
 
-	BS->LocateHandle(ByProtocol, &PxeBaseCodeProtocol, NULL, &size, NULL);
+	uefi_call_wrapper(BS->LocateHandle, 5, ByProtocol, &PxeBaseCodeProtocol, NULL, &size, NULL);
 	if (size == 0) return EFI_UNSUPPORTED; /* no device found, oh well */
 
 	DBG_PRT((L"size=%d", size));
@@ -683,7 +742,7 @@ netfs_install(VOID)
 		return EFI_OUT_OF_RESOURCES;
 	}
 	
-	status = BS->LocateHandle(ByProtocol, &PxeBaseCodeProtocol, NULL, &size, (VOID **)dev_tab);
+	status = uefi_call_wrapper(BS->LocateHandle, 5, ByProtocol, &PxeBaseCodeProtocol, NULL, &size, (VOID **)dev_tab);
 	if (status != EFI_SUCCESS) {
 		ERR_PRT((L"failed to get handles: %r", status));
 		free(dev_tab);
@@ -712,7 +771,7 @@ netfs_uninstall(VOID)
 	for(i=0; i < ndev; i++) {
 		if (dev_tab[i].intf == NULL) continue;
 		nfs = FS_PRIVATE(dev_tab[i].intf);
-		status = BS->UninstallProtocolInterface(nfs->dev, &NetFsProtocol, dev_tab[i].intf);
+		status = uefi_call_wrapper(BS->UninstallProtocolInterface, 3, nfs->dev, &NetFsProtocol, dev_tab[i].intf);
 		if (EFI_ERROR(status)) {
 			ERR_PRT((L"Uninstall %s error: %r", FS_NAME, status));
 			continue;
@@ -725,11 +784,12 @@ netfs_uninstall(VOID)
 		  	FreePool(str);
 			});
 
-		if (nfs->pxe->Mode->Started == TRUE) nfs->pxe->Stop(nfs->pxe);
+		if (nfs->pxe->Mode->Started == TRUE) 
+			uefi_call_wrapper(nfs->pxe->Stop, 1, nfs->pxe);
 
 		free(dev_tab[i].intf);
 	}
-	free(dev_tab);
+	if (dev_tab) free(dev_tab);
 
 	return EFI_SUCCESS;
 }
